        page    64,132
        title   MYAPPL - Example Of REXX Calls To An Application Environment
        comment @

This is MYAPPL.ASM, an example of REXX calls to an application environment

Function:

        Establishes an application environment and runs a REXX program.
        Handles calls to the application environment from the REXX program.

Entry Conditions:

        Run MYAPPL from DOS or a REXX program.  Any parameters passed
        to this program are passed on to the REXX program it calls.

Exit Conditions:

        This program exits with the return code REXX returned.

Include Files:

        REXXCALL.MAC - maps REXX control block
        CALLREXX.ASM - gets address of REXX interrupt handler
        FINDREXX.ASM - gets REXX interrupt number

To recreate MYAPPL.COM (the executable version of this program):

        MASM MYAPPL;
        LINK MYAPPL;
        EXE2BIN MYAPPL MYAPPL.COM

        ERASE MYAPPL.OBJ
        ERASE MYAPPL.EXE

This program was tested with Personal REXX 1.60, DOS 3.3, and MASM 4.0.

@
        page
        include rexxcall.mac    ; map REXX control blocks
        page
sample  segment
        assume  cs:sample,ds:sample
        org     80H
parml   db      ?               ; parameter length
        db      ?               ; separator
parms   db      ?               ; user's parms
        org     100h            ; org past PSP
main:   jmp     begin
                db      20 dup('STACK')
stack           equ     $

;       define control blocks & misc data areas
;       ---------------------------------------

eb              env_block <>            ; environment block
eb_length       equ     $-eb

rc              call_block <>           ; call block
rc_length       equ     $-rc

ab              arg_block <>            ; arg block
ab_length       equ     $-ab

my_name         db      'MACRO'         ; name of REXX program to call
my_name_length  equ     $-my_name

my_env_name     db      'APPLENV  '     ; default environment name
my_extension    db      'APL'           ; default extension

my_result       db      255 dup(?)      ; result (if any) goes here

no_rexx         db      'Error: REXX not loaded.$'
        page
        include callrexx.asm    ; include getint and errint routines
        page
begin   proc    near

;       set up new stack and release unneeded memory
;       --------------------------------------------
        mov     sp,offset stack         ; set up new stack pointer
        mov     bx,offset endcode+15    ; address of end of program
        mov     cl,4
        shr     bx,cl                   ; convert to paragraphs
        mov     ah,4ah                  ; dos function = SETBLOCK
        int     21h                     ; shrink ourselves

;       get address of REXX interrupt handler
;       -------------------------------------
        call    getint                  ; get addr of REXX interrupt handler
        jz      gotint                  ; if no error, continue
        jmp     errint                  ; else print error message and exit
gotint:                                 ; continue ...

;       insure REXX is loaded - abort if not
;       ------------------------------------
        mov     ah,rexx_version         ; function = get Personal REXX version
        mov     al,0                    ; default major version = 0
        pushf                           ; save flags for REXX's iret
        call    callrexx                ; get version number in ax
        or      al,al                   ; did major version change?
        jnz     loaded                  ; yes, then REXX is loaded, so continue
        lea     dx,no_rexx              ; no, get address of error message
        jmp     error                   ; write error message and exit
loaded:

;       Note: during the rest of this routine BX contains segment address
;       -----------------------------------------------------------------
        mov     bx,cs

;       fill in call block
;       ------------------

;       zero call block
        cld
        mov     al,0
        lea     di,rc
        mov     cx,rc_length
        rep     stosb

;       address of arg block
        lea     si,ab
        mov     word ptr rc.arg_list_addr+0,si
        mov     word ptr rc.arg_list_addr+2,bx

;       fill in arg_count, ret_size, and ret_count
        mov     byte ptr rc.arg_count,1
        mov     byte ptr rc.ret_size,255
        mov     byte ptr rc.ret_count,0

;       address of return field
        lea     si,my_result
        mov     word ptr rc.ret_addr+0,si
        mov     word ptr rc.ret_addr+2,bx

;       address of environment block
        lea     si,eb
        mov     word ptr rc.envblock_addr+0,si
        mov     word ptr rc.envblock_addr+2,bx

;       address and length of program name
        lea     si,my_name
        mov     word ptr rc.name_addr+0,si
        mov     word ptr rc.name_addr+2,bx
        mov     rc.name_length,my_name_length
        page
;       fill in environment block
;       -------------------------

;       zero environment block
        mov     al,0            ; fill character
        lea     di,eb           ; address of area to fill
        mov     cx,eb_length    ; length
        rep     stosb           ; zero environment block

;       copy environment name to environment block
        mov     cx,8            ; length of environment name
        lea     si,my_env_name  ; address of environment name
        lea     di,eb.env_name  ; where to put environment name
        rep     movsb           ; copy environment name

;       copy extension to environment block
        mov     cx,3            ; length of extension
        lea     si,my_extension ; address of extension
        lea     di,eb.env_extension ; where to put it
        rep     movsb           ; copy extension

;       set entry point address
        lea     si,applenv
        mov     word ptr eb.env_entry_addr+0,si
        mov     word ptr eb.env_entry_addr+2,bx

;       set misc fields
        mov     eb.env_version,1
        mov     eb.env_dseg,bx
        mov     eb.env_sseg,bx
        mov     ax,sp
        sub     ax,40
        mov     eb.env_sp,ax

;       fill in arg block
;       -----------------

;       zero arg block
        mov     al,0            ; fill character
        lea     di,ab           ; address of area to zero out
        mov     cx,ab_length    ; length
        rep     stosb           ; zero arg block

;       fill in address and length of parms
        lea     si,parms        ; get address of parms
        mov     word ptr ab.arg_addr+0,si
        mov     word ptr ab.arg_addr+2,bx
        mov     al,parml        ; get length of parms
        or      al,al           ; is parm length zero?
        jz      noparms         ; yes, then there were no parms
        dec     al              ; minus 1 for separator
noparms:
        mov     byte ptr ab.arg_length,al

;       set up registers, call MACRO.APL, and exit
;       ------------------------------------------

        mov     ax,REXX_CALL            ; function = call REXX
        mov     bx,cs                   ; get segment address of call_block
        lea     dx,rc                   ; get offset to call_block
        pushf                           ; save flags for REXX's iret
go:     call    callrexx                ; call REXX interrupt handler
        mov     ah,4ch                  ; dos function = exit
        int     21h                     ; return to dos w/REXX's return code
begin   endp
        page

;       This routine handles calls to the APPLENV environment
;       -----------------------------------------------------

;       This example routine simply displays the command string.
;       It exits with a return code of -10
;       On entry, the following registers are of interest:
;              es:ax = pointer to command string
;              cx = length of command string
;              es:si = command environment block

applenv proc   far
; at this point we are using the caller's stack
; but our code and data seg registers are already set up.

        push    bp      ; save callers frame pointer

; switch to our own stack

        mov     di,ss   ; so we can save callers stack regs
        mov     bp,sp   ; ...
        cli             ; disable interrupts
        mov     ss,es:[si].env_sseg ; get stack seg
        mov     sp,es:[si].env_sp   ; set stack pointer
        sti             ; allow interrupts
        push    bp      ; save callers sp
        push    di      ; save callers ss
        mov     bp,sp   ; remember position in stack

; display message and set return code
        push    es
        pop     ds      ; get segment address of parm
        mov     dx,ax   ; get offset to parm
        mov     bx,1    ; handle = stdout
        mov     ah,40h  ; dos function = write to file or device
        int     21h     ; call dos
        mov     ax,-10  ; set return code

; restore the caller's stack

        pop     di      ; caller's ss
        pop     si      ; caller's sp
        cli             ; prevent interrupts
        mov     ss,di   ; restore ss
        mov     sp,si   ; restore sp
        sti             ; allow interrupts
        pop     bp      ; restore frame pointer
        ret
applenv endp

;       Error exit routine - DX has offset of error message.
;       ----------------------------------------------------

error   proc    near
        mov     ah,9    ; dos function = print string
        int     21h     ; call dos to print error message
        ret             ; return to dos
error   endp

endcode equ     $

sample  ends
        end  main
