         page 58,132
;------------------------------------------------------------------------------
;     This subroutine is a gate for calls from Personal REXX to a
; resident environment command handler.
;
;     It is coded for Microsoft C, small, medium, or large model.
;
;     The address of this routine should be placed in the env_entry_address
; field of the environment block used to register the command handler.
;
;     It sets up segment registers, etc. for a C language command handler,
; calls the command handler, then returns to Personal REXX.
;
;     It is called from REXX as a far procedure.
;
;     The C command handler should use the standard C linkage conventions,
; so it should be declared (or default) to _cdecl type.
;
;     For all models, the 4-byte address of the command string passed from
; REXX is put on the stack as argument to command handler, followed by the
; 2-byte length of the string.
;
;     The full C prototype of the command handler should be:
;
;  int _cdecl command_handler(
;        char far *command,
;        int  length);
;
;     The address of this C command handler routine should be in the
; env_exec_addr field of the environment block.
;
;     For small model, environment block field env_exec_cseg should be 0,
; and a near call to function handler will be issued.
;
;     For medium and large models, a far call is used.
;------------------------------------------------------------------------------

          include rexxcall.mac ; define an environment block

          public _rexxcall
         .MODEL SMALL
         .CODE
_rexxcall proc far

; on entry, the following registers are of interest:
;        es:ax = pointer to command string
;        cx = length of command string
;        es:si = command environment block
; at this point we are using the caller's stack
; but our code and data seg registers are already set up.

         push  bp             ; save callers frame pointer

; switch to our own stack

         mov   di,ss          ; so we can save callers stack regs
         mov   bp,sp          ; ...
         cli                  ; disable interrupts
         mov   ss,es:[si].env_sseg ; get stack seg
         mov   sp,es:[si].env_sp   ; set stack pointer
         sti                  ; allow interrupts
         push  bp             ; save callers sp
         push  di             ; save callers ss
         mov   bp,sp          ; remember position in stack

; setup arguments to command environment
         push  cx             ; push length of command string
         push  es             ; push segment, offset of command string
         push  ax

; ready to call command executor

         cmp   word ptr es:[si].env_exec_addr+2,0
         je    near_call      ; use near call if command handler segment is 0
         call  dword ptr es:[si].env_exec_addr ; far call to command handler
         jmp   chop
near_call:
         call  word ptr es:[si].env_exec_addr  ; near call to command handler
chop:
         mov   sp,bp          ; chop stack

; return, with ax containing a return code
; restore the caller's stack

         pop   di             ; caller's ss
         pop   si             ; caller's sp
         cli                  ; prevent interrupts
         mov   ss,di          ; restore ss
         mov   sp,si          ; restore sp
         sti                  ; allow interrupts
         pop   bp             ; restore frame pointer
         ret                  ; return

_rexxcall endp
        end
