/* Data for calling Personal REXX from an Application Program */

#pragma pack(1)

#ifndef FARP                                    /* far pointer definition */
#define FARP char far *                         /* default for Microsoft C */
#endif

/* Personal REXX Interrupt Functions and Sub-Functions */

#define REXX_CALL       0x00  /* call a REXX program as a macro */
#define REXX_SHV        0x01  /* shared variable interface request */
#define REXX_FCN        0x02  /* install or uninstall a function package */
#define FCN_INSTALL     0x01  /* sub-function: install a function package */
#define FCN_UNINSTALL   0x02  /* sub-function: uninstall a function package */
#define REXX_UNLOAD     0x04  /* unload resident REXX interpreter */
#define REXX_VERSION    0x05  /* get Personal REXX version number */
#define REXX_SIGNAL     0x06  /* signal a Personal REXX condition */

#define STK_WRITE       0x70  /* write char to program stack */
#define WRITE_FIFO      0x00  /* sub-function: write FIFO */
#define WRITE_LIFO      0x01  /* sub-function: write LIFO */
#define STK_BYPASS      0x71  /* read from keyboard buffer (bypass stack) */
#define STK_READ        0x72  /* return next char, if none return status */
#define STK_MAKEBUF     0x73  /* make a new program stack buffer */
#define STK_DROPBUF     0x74  /* drop a program stack buffer */
#define DROPBUF_CURRENT 0x00  /* sub-function: drop current stack buffer only */
#define DROPBUF_BUFFNO  0x01  /* sub-function: drop specific & higher buffers */
#define STK_USED        0x75  /* count characters currently stacked */
#define STK_SET_STATUS  0x76  /* enable or disable the program stack */
#define SET_DISABLE     0x00  /* sub-function: disable the program stack */
#define SET_ENABLE      0x01  /* sub-function: enable the program stack */
#define STK_SENTRIES    0x77  /* count lines in program stack */
#define STK_STATUS      0x78  /* determine stack status (enabled or disabled) */
#define STK_BUFFNO      0x79  /* retrieve current program buffer number */
#define STK_FREE        0x7A  /* retrieve available space in stack (in chars) */

/* Personal REXX Control Block Definitions */

/* Argument Block */
struct arg_block {
    FARP arg_addr;                      /* ptr to an arg string */
    long arg_length; };                 /* length of an arg string */

/* Break Block */
struct break_block {
    unsigned break_env;                 /* ctrl-break in application */
    unsigned break_rexx; };             /* ctrl-break in rexx */

/* Call Block */
struct call_block {
    FARP arg_list_addr;                 /* ptrs to arg list */
    long arg_count;                     /* # of args */
    long ret_size;                      /* max length of answer */
    FARP ret_addr;                      /* where to put answer */
    long ret_count;                     /* actual length of answer */
    FARP envblock_addr;                 /* ptr to env_block */
    FARP name_addr;                     /* ptr to program name */
    unsigned name_length;               /* length of program name */
    FARP break_addr;                    /* ptr to break_block */
    unsigned call_status;               /* termination code */
    unsigned char call_rsv1;            /* must be 0 */
    unsigned char exec_flags;           /* control compiler operation */
    unsigned char exec_flags_off;       /* exec flags to turn off */
    unsigned char misc_flags;           /* miscellaneous flags */
    unsigned char misc_flags_off;       /* misc flags to turn off */
    unsigned char trace_string[4];      /* initial trace string */
    unsigned char call_rsv2;            /* must be 0 */
    unsigned call_rsv3;                 /* must be 0 */
    };

/* exec_flags definitions */
#define F_EXECUTE       0x80
#define F_OBJECT_CODE   0x40
#define F_SAVE_SOURCE   0x20
#define F_SAVE_STMT_OBJ 0x10
#define F_SAVE_STMT_XEQ 0x08
#define F_TRACE_ON      0x04

/* bits in misc_flags */
#define M_USE_EMS       0x80

/* Environment Block */
struct env_block {
    FARP env_rsv1;                      /* must be 0 */
    unsigned char env_name[8];          /* name of environment */
    unsigned char env_extension[3];     /* default extension */
    FARP env_entry_addr;                /* ptr to env entry point */
    FARP env_exec_addr;                 /* ptr to env dispatcher */
    unsigned env_dseg, env_sseg, env_sp;/* ds, ss, and sp of env */
    FARP env_path_addr;                 /* ptr to env default path */
    unsigned env_path_length;           /* length of env path */
    unsigned char env_flag;             /* env_block flags */
    unsigned char env_version;          /* env_block version number (1) */
    FARP env_break_addr;                /* ptr to break block */
    unsigned env_service_type;          /* service types */
    FARP env_service_addr;              /* service entry point */
    unsigned char env_rsv2;             /* must be 0 */
    unsigned env_rsv3;                  /* must be 0 */
    unsigned env_rsv4;                  /* must be 0 */
    };

/* bits in env_flag */
#define ENV_FLAG_RESIDENT 0x40          /* resident environment */

/* service types */
#define OUTPUT_SERVICE   0x8000
#define INPUT_SERVICE    0x4000
#define FILE_SERVICE     0x2000
#define EXIT_SERVICE     0x1000

/* Shared Variable Request Block */
struct shvblock {
    FARP shvnext_addr;                  /* ptr to next shvblock */
    long shvuser;                       /* shvblock user field */
    char shvcode;                       /* shv function code */
    char shvret;                        /* shv return code */
    int  shvrsv1;                       /* must be 0 */
    long shvbufl;                       /* length of fetch buffer */
    FARP shvnam_addr;                   /* ptr to variable name */
    long shvnaml;                       /* length of variable name */
    FARP shvval_addr;                   /* ptr to variable value */
    long shvvall;                       /* length of variable value */
    };

/* Shared Variable Interface Function Codes (shvcode) */
#define SHVSET      'S'                 /* set variable from given value */
#define SHVFETCH    'F'                 /* copy value of variable to buffer */
#define SHVDROPV    'D'                 /* drop variable name */
#define SHVSYSET    's'                 /* symbolic name set variable */
#define SHVSYFET    'f'                 /* symbolic name fetch variable */
#define SHVSYDRO    'd'                 /* symbolic name drop variable */
#define SHVNEXTV    'N'                 /* get next variable */
#define SHVPRIV     'P'                 /* fetch private information */

/* Shared Variable Interface Return Code Flags (shvret) */
#define SHVCLEAN    0x00                /* execution was ok */
#define SHVNEWV     0x01                /* variable did not exist */
#define SHVLVAR     0x02                /* last variable transferred */
#define SHVTRUNC    0x04                /* truncation occurred during "fetch" */
#define SHVBADN     0x08                /* invalid variable name */
#define SHVBADF     0x80                /* invalid function code (shvcode) */

#pragma pack()
