         page 58,132
;------------------------------------------------------------------------------
; This subroutine is a gate for calls from the REXX interpreter to service
;  exit routines.
;
; This version is coded for Microsoft C 5.0 or later, small, medium, or
;  large model.
;
; The address of this routine should be saved in the environment block
;  in the env_service_addr field.
;
; It sets up the stack, etc. for a C language service handler,
;  calls the service handler, then returns to Personal REXX.
;
; It is called from REXX as a far procedure.
;
; The C service routine should use the standard C linkage conventions, so
;  it should be declared (or default) to _cdecl type.
;
; The full C prototype of the service handler should be:
;
;  int _cdecl service_routine(
;        int  type,
;        char far *buffer,
;        int  far *length,
;        int  line_number,
;        char far *file_name);
;
; The address of the C service handler routine should be in the
;  env_exec_addr field of the environment block.
;
; For small model, the environment block field env_exec_cseg should be 0,
;  and a near call to the service handler will be issued.
;
; For medium and large models, a far call is used.
;------------------------------------------------------------------------------

         include rexxcall.mac ; define an environment block

         public _rexxserv
         .MODEL SMALL
         .CODE

temp     dw    ?              ; scratch storage

_rexxserv proc far

; at this point we are using the caller's stack
; but our code and data seg registers are already set up.

         push  bp             ; save callers frame pointer

; on entry, the following registers are of interest:
;        ax = type of service call
;              1 = output service (screen)
;              2 = input service (keyboard)
;              3 = source file reading
;        cx = length of buffer
;        dx = file line number (ax = 3 only)
;        es:bx = address of I/O buffer
;        es:di = address of file name (null terminated) (ax = 3 only)
;        es:si = address of environment block

; first, switch to our own stack

         mov   temp,ss        ; so we can save callers stack regs
         mov   bp,sp          ; ...
         cli                  ; disable interrupts
         mov   ss,es:[si].env_sseg ; get stack seg
         mov   sp,es:[si].env_sp   ; set stack pointer
         sti                  ; allow interrupts
         push  bp             ; save callers sp
         push  temp           ; save callers ss
         mov   bp,sp          ; remember position in stack
         sub   sp, 2          ; allow space for local variables
                              ; (buffer length)

; ready to call service handler
; C calling conventions specifies parameters are pushed right to left

         push  es             ; file name
         push  di
         push  dx             ; line number
         mov   word ptr [bp-2], cx ; save buffer length on stack
         lea   cx, [bp-2]          ; point at buffer length
         push  ss             ; address of buffer length
         push  cx
         push  es             ; I/O buffer address
         push  bx
         push  ax             ; service exit type

         cmp   word ptr es:[si].env_exec_addr+2,0
         je    near_call      ; use near call if service handler segment is 0
         call  dword ptr es:[si].env_exec_addr ; far call to service handler
         jmp   chop
near_call:
         call  word ptr es:[si].env_exec_addr  ; near call to service handler
chop:
         mov   sp,bp          ; chop stack
         mov   cx, word ptr [bp-2] ; returned line length

; return, with ax containing a return code
; restore the caller's stack

         pop   di             ; caller's ss
         pop   si             ; caller's sp
         cli                  ; prevent interrupts
         mov   ss,di          ; restore ss
         mov   sp,si          ; restore sp
         sti                  ; allow interrupts
         pop   bp             ; restore frame pointer
         ret                  ; return

_rexxserv endp

_text   ends
        end
