/*****************************************************************************/
/* CMDFILE filter:                                                           */
/*                                                                           */
/* Helps create 'CMS EXEC'-like .REX files. CMDFILE takes two nonnegative    */
/* numbers as parameters. The first number specifies how many command file   */
/* parameters precede each line of data passed to CMDFILE, and the second    */
/* number specifies how many command parameters follow each line of data     */
/* passed to CMDFILE. For example,                                           */
/*     listfile *.lst *.doc (fileid | rexx cmdfile 1 1 >many.rex             */
/* might write to MANY.REX the lines                                         */
/*    %1 C:\COMMON\FILE1.LST %2                                              */
/*    %1 C:\COMMON\FILE2.LST %2                                              */
/*    %1 C:\COMMON\FILE3.LST %2                                              */
/*    %1 C:\COMMON\FILE1.DOC %2                                              */
/*    %1 C:\COMMON\FILE2.DOC %2                                              */
/*                                                                           */
/* You could then use MANY.REX to enter system commands against the files    */
/* listed in MANY.REX.  For instance, in this example                        */
/*    many copy d:                                                           */
/* would copy all files listed in MANY.REX to the D: drive.                  */
/*****************************************************************************/

parse arg prefix suffix .

/* Defaults if appropriate */
if prefix = "" & suffix = "" then prefix = 2
if suffix = "" then suffix = 0
if prefix = "." | prefix = "%" then prefix = 0

if \datatype(prefix,'W') | \datatype(suffix,'W') then do
   say 'Error: Incorrect data type of parameter references'
   exit
   end
if prefix < 0 | suffix < 0 | prefix+suffix > 9 then do
   say 'Error: Parameter references out of range.'
   exit
   end

last = prefix + suffix
argnumber = 1
front = ""
back = ""

do argnumber = 1 to prefix
   front = front "%"argnumber
   end
do argnumber = prefix + 1 to last
   back = back "%"argnumber
   end
do while lines() \= 0
   line = front linein() back
   call lineout , line
   end
call lineout
